tree grammar Language4_AST_eval;

options
{
   tokenVocab=Language4_AST;
   ASTLabelType=CommonTree;
}

@header
{
import java.util.HashMap;
}

@members
{
/** for storing <variable, value> pairs */
HashMap environment = new HashMap();
}

expression returns [Value result]
           : exp EOF {$result=$exp.result;}
           ;

exp returns [Value result]
    : ^('begin' a=exp {$result=a;} (b=exp {$result=b;})* )
    | ^('var' VARIABLE a=exp) {environment.put($VARIABLE.text, a);
                               $result = a;}
    | ^('+'  a=exp b=exp)   {$result = new Value("int", a.valueI+b.valueI);}
    | ^('-'  a=exp (b=exp   {$result = new Value("int", a.valueI-b.valueI);}
                   |        {$result = new Value("int", -a.valueI);}
                   )    )
    | ^('*'  a=exp b=exp)   {$result = new Value("int", a.valueI*b.valueI);}
    | ^('/'  a=exp b=exp)   {$result = new Value("int", a.valueI/b.valueI);}
    | ^('%'  a=exp b=exp)   {$result = new Value("int", a.valueI\%b.valueI);}
    | ^('^'  a=exp b=exp)   {$result = new Value("int", (int)Math.pow(a.valueI,b.valueI));}
    | ^('&&' a=exp b=exp)   {$result = new Value("bool", a.valueB&&b.valueB);}
    | ^('||' a=exp b=exp)   {$result = new Value("bool", a.valueB||b.valueB);}
    | ^('!'  a=exp      )   {$result = new Value("bool", !a.valueB);}
    | ^('<'  a=exp b=exp)   {$result = new Value("bool", a.valueI<b.valueI);}
    | ^('>'  a=exp b=exp)   {$result = new Value("bool", a.valueI>b.valueI);}
    | ^('<=' a=exp b=exp)   {$result = new Value("bool", a.valueI<=b.valueI);}
    | ^('>=' a=exp b=exp)   {$result = new Value("bool", a.valueI>=b.valueI);}
    | ^('==' a=exp b=exp)   {$result = new Value("bool",  a.equals(b));}
    | ^('!=' a=exp b=exp)   {$result = new Value("bool", !a.equals(b));}
    | INTEGER  {$result = new Value("int",  Integer.parseInt($INTEGER.text));}
    | BOOLEAN  {$result = new Value("bool", $BOOLEAN.text.equals("true"));}
    | VARIABLE {$result = (Value)environment.get($VARIABLE.text);
                if ( result == null )
                   System.err.println("undefined variable "+$VARIABLE.text);}
    ;
