/*
  This program tests the lexers and parsers created
  by Antlr from the grammars Language4_AST.g and Language4_AST_eval.g

  See the following page for information on using Antlr
  on the command line to create the lexer and parser, to
  compile the lexer and parser, and then to compile and
  run this program.
  http://www.antlr.org/wiki/pages/viewpage.action?pageId=729
*/
import org.antlr.runtime.*;
import org.antlr.runtime.tree.*;

public class TestParser_AST_eval
{
   public static void main(String[] args)
   {
      String[] exprs =
      {
        "(begin " +
           "(begin (var x 2)" +
                    "(var y 2))" +
            "(var z (+ x y))" +
        ")",
        "(begin " +
           "(var x 4) " +
           "(begin (var y false)" +
                   "(|| y (== (+ 2 x) (* 2 3)))))",
        "( begin " +
           "(var x (begin (var z 2) (+ z z)))" +
            "(begin (var y false) " +
                   "(begin (var u 2) " +
                           "(begin " +
                              "(var v (|| y (== (+ u 4) (* u 3))))" +
                              "v))))",
        "(var x " +
           "(begin (var y 2)" +
                   "(var z (+ y 3))))",
        // illegal expressions
        "(var z (+ (var x (var y 2)) (+ y (* 3 x))))",
        "(begin (var x false ) (+ 1 x))",
        "(+ (!= (* 2 2) 5) (/ 1 0))"
      };

      for (int i = 0; i < exprs.length; i++)
      {
         Language4_ASTLexer lexer = new Language4_ASTLexer(new ANTLRStringStream(exprs[i]));
         Language4_ASTParser parser = new Language4_ASTParser(new CommonTokenStream(lexer));

         System.out.println( "Parsing: " + exprs[i] );
         try
         {
            // The result from this parse is a tree.
            Language4_ASTParser.expression_return r = parser.expression();
            CommonTree tree = (CommonTree)r.getTree();
            System.out.println( tree.toStringTree() );

            // Walk the resulting tree.
            // Create a tree node stream from the resulting tree.
            CommonTreeNodeStream nodes = new CommonTreeNodeStream(tree);
            // Create a tree parser.
            Language4_AST_eval walker = new Language4_AST_eval(nodes);
            // Launch the tree walker at the start rule "expression".
            Value v = walker.expression();
            System.out.println("result = " + v);
         }
         catch (Exception e)
         {
            System.out.println(e);
         }
         System.out.println("========================================");
      }
   }//main

}//TestParser_AST_eval
